#!/bin/sh

# Setup script for shell configuration files.

# Copyright (C) 2011-2018  Simon Ruderich
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


set -eu

. ../lib.sh


# Helper functions.
terminal_info() {
    infocmp "$@" 2>&1
}
terminal_available() {
    terminal_info "$@" > /dev/null
}
# Usage: <file> <name> <grep-string> <value>
apply_optional_replacement() {
    if test -n "$4"; then
        printf '%s: using %s %s\n' "$1" "$2" "$4"
        generate "$1" '' simple_cpp \
            "$3" -- "$4"
    else
        printf '%s: removing %s display\n' "$1" "$2"
        grep_i -v "$3" "$1"
    fi
}
# Check if `infocmp` is available.
if ! infocmp >/dev/null 2>&1; then
    echo 'Warning: `infocmp` not available! 256color checks will fail.'
    echo
fi


# DIRECTORY SETUP

# Create private temporary directory used by many tools (including GNU screen
# and tmux).
mkdir -p ~/.tmp
chmod 0700 ~/.tmp

# Create rlwrap history directory.
mkdir -p shell/rlwrap
# Create zsh cache directory.
mkdir -p zsh/cache


# FILE SETUP

# Generate ~/.less with lesskey. Prevent cluttering ~/ by storing the history
# file in this directory; this requires replacing the constant HISTORY_PATH in
# lesskey.
echo 'lesskey: generating .lesskey'
simple_cpp <lesskey \
    HISTORY_PATH -- "`pwd`/lesshistory" \
    | lesskey -
chmod 0600 ~/.less

# Custom colors for GNU ls.
if installed dircolors; then
    echo '# WARNING! DO NOT EDIT THIS FILE!' >shell/dircolors
    dircolors -b shell/dircolors.in >>shell/dircolors
fi

# Find the required options to get colored ls output. GNU ls is preferred. See
# shell/aliases.in for details. Doing this here instead of in shell/aliases
# speeds up shell starts.
#
# GNU ls with colors available.
if ls --color >/dev/null 2>&1; then
    ls_args='command ls --color'
# Normal (BSD) ls with colors available.
elif ls -G >/dev/null 2>&1; then
    # Don't display hidden files by default when running as root (-I), I use
    # `la` for that.
    ls_args='CLICOLOR_FORCE=1 command ls -G -I'
# OpenBSD has no colored ls, use colorls instead.
elif colorls -G >/dev/null 2>&1; then
    ls_args='CLICOLOR_FORCE=1 colorls -G'
# Simple ls with no colors.
else
    ls_args='command ls'
fi

# Also check if `column -t` is available.
if echo test | column -t >/dev/null 2>&1; then
    column=' | column -t'
else
    column=
fi

generate shell/aliases .in simple_cpp \
    LS_ARGS COLUMN -- \
    "$ls_args" "$column"

# If `tig` is not available use my simple replacement.
if ! installed tig; then
    echo "alias tig='git tig'" >>shell/aliases
fi

# Check if grep supports --color=auto.
if echo test | grep --color=auto test >/dev/null 2>&1; then
    :
else
    echo 'shell/aliases: removing grep --color=auto'
    sed_i '/^alias grep=/ s/^/#/' shell/aliases
fi

generate screenrc .in cat
# As screen-256color is not widely supported use it only on machines where the
# matching terminfo entry is available. This also requires a terminal emulator
# which supports 256 colors. Also used for tmux.
use_256colors=
if terminal_available screen-256color; then
    # GNU/Linux's virtual terminal doesn't support 256 colors. If setup.sh is
    # run one one, assume this user is mostly used from the terminal.
    if test x"$TERM" = xlinux || test x"$TERM" = xscreen.linux; then
        echo 'screenrc: running on virtual terminal, disabling 256 colors'
    # Called through SSH connection, assume the local system supports 256
    # colors.
    elif test -n "${SSH_CONNECTION:+set}"; then
        use_256colors=1
    # We have rxvt-unicode installed, check if it supports 256 colors.
    elif installed urxvt; then
        # Thanks to deryni in #rxvt-unicode on Freenode (2012-10-14 22:54
        # CEST) for the strings/grep idea. The grep check is for "correct" 256
        # rxvt-unicode binaries (e.g. Debian's rxvt-unicode-256color), the
        # terminal_info check for manual installations which modify
        # rxvt-unicode's terminfo entry.
        urxvt_path=`installed_path urxvt`
        urxvt_grep=`strings "$urxvt_path" | grep '^TERM=rxvt-'`
        if test x"$urxvt_grep" = 'xTERM=rxvt-unicode-256color' \
                || terminal_info rxvt-unicode \
                    | grep -F 'colors#256' >/dev/null; then
            use_256colors=1
        fi
    # Check if XTerm supports 256 colors (not a perfect check, but most XTerm
    # support 256 colors).
    elif terminal_available xterm-256color; then
        use_256colors=1
    fi
fi
if test -z "$use_256colors"; then
    echo 'screenrc: removing 256 colors'
    sed_i 's/Enable 256 color/Disable 256 color/;
           s/screen-256color/screen/' screenrc
fi
# Rxvt doesn't need the attrcolor "fix". As I prefer rxvt assume I use it when
# it's installed.
if installed rxvt; then
    echo 'screenrc: removing attrcolor "fix"'
    sed_i 's/attrcolor b ".I"/#attrcolor b ".I"/' screenrc
fi
# Display current battery charge on computers with a battery. Necessary lines
# are marked as "(BATTERY)". Also used for Tmux.
battery=
for x in /sys/class/power_supply/BAT*; do
    test -d "$x" || continue
    battery="$x"
done
apply_optional_replacement screenrc \
    battery BATTERY "$battery"
# Display current temperature. Necessary lines are marked as "(TEMPERATURE)".
# Also used for Tmux.
temperature=/sys/devices/platform/coretemp.0
if ! test -d "$temperature"; then
    temperature=
fi
apply_optional_replacement screenrc \
    temperature TEMPERATURE "$temperature"
# Settings for local hosts.
if test -z "${SSH_CONNECTION:+set}"; then
    echo 'screenrc: removing remote options'
    grep_i -v '(REMOTE)' screenrc
fi


if installed tmux; then
    generate tmux.conf .in cat

    apply_optional_replacement tmux.conf \
        battery BATTERY "$battery"
    apply_optional_replacement tmux.conf \
        temperature TEMPERATURE "$temperature"

    # Old Tmux versions can't handle that.
    generate tmux.conf '' ./bin/remove-continuation.pl

    # Add mappings to switch to windows 10-29 quickly. See tmux-window.pl for
    # details.
    perl ./tmux-window.pl 1 "`pwd`/tmux-window2.conf" >tmux-window1.conf
    perl ./tmux-window.pl 2                           >tmux-window2.conf
    # Set absolute path to tmux-window1.conf in tmux.conf.
    generate tmux.conf '' simple_cpp \
        TMUX_WINDOW_PATH -- "`pwd`/tmux-window1.conf"

    # 256 colors not available.
    if test -z "$use_256colors"; then
        echo 'tmux.conf: removing 256 colors'
        sed_i 's/Enable 256 color/Disable 256 color/;
               s/screen-256color/screen/' tmux.conf
    fi
    # Tmux doesn't display a warning if the shell wasn't found!
    if test ! -x '/bin/zsh'; then
        echo 'tmux.conf: removing /bin/zsh as shell'
        sed_i 's/zsh/sh/' tmux.conf
    fi
fi

# Htop overwrites the comments in its configuration file.
generate htoprc .in cat

if installed dig; then
    # dig doesn't support any comments in digrc.
    grep -v -E '^#' digrc.in >digrc
fi


# LINK SETUP

# Link setup for shells.
link shell ~/.shell
link bash ~/.bash
link bash/rc ~/.bashrc
link bash/profile ~/.bash_profile
link bash/logout ~/.bash_logout
if installed csh; then
    link csh/rc ~/.cshrc
fi
link zsh ~/.zsh
link zsh/env ~/.zshenv
link zsh/rc ~/.zshrc
link zsh/logout ~/.zlogout

if installed tmux; then
    link terminfo ~/.terminfo
fi

# Link setup for additional files.
if installed crontab; then
    link crontab.d ~/.crontab.d
fi
link lessfilter ~/.lessfilter
if installed colordiff; then
    link colordiffrc ~/.colordiffrc
fi
link inputrc ~/.inputrc
if installed ghci; then
    link haskeline ~/.haskeline
fi
link screenrc ~/.screenrc
if installed tmux; then
    link tmux.conf ~/.tmux.conf
fi
if installed htop; then
    link htoprc ~/.htoprc
    # New location for htoprc. Use both for compatibility.
    mkdir -p ~/.config/htop
    link htoprc ~/.config/htop/htoprc
fi
if test -d ~/.ssh && test -O ~/.ssh; then
    mkdir -p ~/.ssh/master
    link ssh_config ~/.ssh/config
fi
if installed dig; then
    link digrc ~/.digrc
fi
